#!/bin/sh
# command name(voip_sig.bat/auto_sig.bat)
# $1: up/down

[ "X${X7DBG}" = "Xon" ] && set -x

. /etc/ibsbat/ibs.subr
. /etc/rc.conf

CMDBAT=`basename $0`
LOCKFILE=/var/lock/subsys/voip_sig.lock
DHCPC_INFO=/etc/dhcpc/dhcpcd-eth0.info

unset TOP
AUTOUP_FILE=${TOP}/etc/ibsbat/Router/ipaddress_of_wan/autoup.txt
AUTOUP_LOCKCMD=${TOP}/etc/ibsbat/autouplockf_sub.bat
HOSTS_LOCKCMD=${TOP}/etc/ibsbat/hostslockf_sub.bat
WINFO_AUTOUP_UP=${TOP}/tmp/autoup_conn
WINFO_AUTOUP_DW=${TOP}/tmp/autoup_idle
WINFO_HOSTS_UP=${TOP}/tmp/voip_conn
WINFO_HOSTS_DW=${TOP}/tmp/voip_idle

LOCK_TOUT=30

sig_syslog()
{
	logger -p kern.info -t $CMDBAT $1
}

read_dhcpc_info()
{
	unset IPADDR
	if [ -f "${DHCPC_INFO}" ]; then
		sig_syslog "Found ${DHCPC_INFO}"
		. ${DHCPC_INFO}
	else
		sig_syslog "Not found ${DHCPC_INFO}"
		IPADDR=`ip -f inet addr show dev eth0 | sed -n '2p' | awk '{printf"%s\n",$2}'`
		IPADDR=${IPADDR%\/*}
	fi
}

#$1: up/down
send_signal()
{
	local sigtype=
	[ -n "${1}" ] || return 1
	echo "Send signal"
	if [ -f /var/log/ibs/ibsproc_service.pid ]; then
		case $1 in
		up)   sigtype=SIGUSR1 ;;
		down) sigtype=SIGUSR2 ;;
		esac
		_pid=`cat /var/log/ibs/ibsproc_service.pid`
		if [ "X$_pid" != "X" ]; then
			sig_syslog "send signal ${sigtype} service pid."
			kill -${sigtype} $_pid
		fi
	else
		sig_syslog "service pid nod found."
	fi
	return 0
}

edit_autoup()
{
	case $1 in
		up)
			[ ! -f ${AUTOUP_FILE} ] && return 1

			_type=`sed -n '1p' ${AUTOUP_FILE}`
			sig_syslog "autoup entry:${_type}"
			case $_type in
				ctu_ip|ctu_dhcp)
					sig_syslog "autoup entry found."
					read_dhcpc_info
					echo "$_type" > ${AUTOUP_FILE}

					if [ -n "$IPADDR" ]; then
						echo "$IPADDR ibs-autoup" >> ${AUTOUP_FILE}
					fi
					touch ${WINFO_AUTOUP_UP}
					chown ibsuser:ibsuser ${WINFO_AUTOUP_UP}
					send_signal up
					;;
				*) sig_syslog "entry not found." ;;
			esac
			;;
		down)
			[ ! -f ${AUTOUP_FILE} ] && return 1

			_type0=`sed -n '1p' ${AUTOUP_FILE}`
			_type=${_type0###}
			sig_syslog "autoup entry:${_type0} ${_type}"
			case $_type in
				ctu_ip|ctu_dhcp)
					sig_syslog "autoup entry found."
					echo "${_type0}" > ${AUTOUP_FILE}
					touch ${WINFO_AUTOUP_DW}
					chown ibsuser:ibsuser ${WINFO_AUTOUP_DW}
					send_signal down
					;;
				*) sig_syslog "entry not found." ;;
			esac
			;;
		*) ;;
	esac
}

edit_hosts()
{
	case $1 in
		up)
			read_dhcpc_info
			_hosts=`grep ibswan /etc/hosts`
			if [ $? = 0 ] ; then
				cat /etc/hosts | sort | uniq | grep -v ibswan > /etc/hosts.up
				cat /etc/hosts | sort | uniq | awk -v _localip=$IPADDR '/ibswan/{printf"%s  ibswan\n",_localip}' >> /etc/hosts.up
				mv /etc/hosts.up /etc/hosts
				sig_syslog "edit_hosts ibswan set.(${IPADDR})"
			else
				cat /etc/hosts | sort | uniq > /etc/hosts.up
				echo "$IPADDR ibswan" >> /etc/hosts.up
				mv /etc/hosts.up /etc/hosts
				sig_syslog "edit_hosts ibswan set.(${IPADDR})"
			fi

			touch ${WINFO_HOSTS_UP}
			chown ibsuser:ibsuser ${WINFO_HOSTS_UP}
			#unlock$B$N8e$,NI$$$+!)(B
			send_signal up
			;;
		down)
			_hosts=`grep ibswan /etc/hosts`
			if [ $? = 0 ] ; then
				cat /etc/hosts | grep -v ibswan > /etc/hosts.down
				mv /etc/hosts.down /etc/hosts
				touch ${WINFO_HOSTS_DW}
				chown ibsuser:ibsuser ${WINFO_HOSTS_DW}
				ipdown_syslog "edit_hosts ibswan remove."
				#unlock$B$N8e$,NI$$$+!)(B
				#send_signal down
				send_signal up
			fi
			;;
		*) ;;
	esac
}

voip_sig()
{
	local _comd=$1
	local _vtype=
	check_iptell_type
	_vtype=$?

	if [ "X$_vtype" = "X6" ]; then
		${HOSTS_LOCKCMD} ppp lock
		local _lock_ret=$?
		case $_lock_ret in
			0|3)
				if [ $_lock_ret = 3 ]; then
					sig_syslog "hostslockf: double lock"
				fi
				edit_hosts $_comd
				${HOSTS_LOCKCMD} ppp unlock
				;;
			1) ;;
			2)
				#$B%m%C%/BT$A(B
				local _loop_lock=0
				until /etc/ibsbat/hostslockf_sub.bat ppp check 2>&- ; do
				sleep 1
					_loop_lock=$(( $_loop_lock + 1 ))
					if [ $_loop_lock -ge "${LOCK_TOUT}" ]; then
						break
					fi
				done
				if [ $_loop_lock -ge ${LOCK_TOUT} ]; then
					sig_syslog "hostslock time out"
				else
					edit_hosts $_comd
				fi
				${HOSTS_LOCKCMD} ppp unlock
				;;
			*) sig_syslog "hostslockf return code unknown" ;;
		esac
	fi
}

auto_sig()
{
	local _comd=$1
	local _atype=
	check_autoup_type
	_atype=$?

	if [ "X$_atype" = "X6" ]; then
		${AUTOUP_LOCKCMD} ppp lock
		_lock_ret=$?
		case $_lock_ret in
			0|3)
				if [ $_lock_ret = 3 ]; then
					sig_syslog "autouplockf: double lock"
				fi
				edit_autoup  $_comd
				${AUTOUP_LOCKCMD} ppp unlock
				;;
			1) ;;
			2)
				_loop_lock=0
				until /etc/ibsbat/autouplockf_sub.bat ppp check 2>&- ; do
					sleep 1
					_loop_lock=$(( $_loop_lock + 1 ))
					if [ $_loop_lock -ge "${LOCK_TOUT}" ]; then
						break
					fi
				done
				if [ $_loop_lock -ge ${LOCK_TOUT} ]; then
					sig_syslog "autouplock time out"
				else
					edit_autoup $_comd
				fi
				${AUTOUP_LOCKCMD} ppp unlock
				;;
			*) sig_syslog "autouplockf return code unknown" ;;
		esac
	fi
}

#####
sig_syslog "exec."
ln -s $0 ${LOCKFILE}
if [ $? != 0 ]; then
	sig_syslog "Already executing $CMDBAT"
	exit 2
fi
trap "rm -f ${LOCKFILE}" EXIT

if [ ! -n "${1}" ]; then
	sig_syslog "arg1 is not define."
	exit 2
fi

case $CMDBAT in
voip_sig.bat)
	if [ "X$1" = "Xup" ]; then
		voip_sig up
		RETVAL=0
	elif [ "X$1" = "Xdown" ]; then
		voip_sig down
		RETVAL=0
	else
		RETVAL=1
	fi
	;;
auto_sig.bat)
	if [ "X$1" = "Xup" ]; then
		auto_sig up
		RETVAL=0
	elif [ "X$1" = "Xdown" ]; then
		auto_sig down
		RETVAL=0
	else
		RETVAL=1
	fi
	;;
*) RETVAL=1 ;;
esac

exit $RETVAL
