#!/bin/sh

DEBUG=1
TOP=/tmp/work

LOCK_EXP=".ibs-soho.lock"

lan_ifname=tlp1
wan_ifname=tlp0

RC_CONF=${TOP}/etc/rc.conf
RC_CONF_LOCK=${RC_CONF}${LOCK_EXP}

DHCPD_CONF=${TOP}/etc/dhcpd.conf
DHCPD_CONF_LOCK=${DHCPD_CONF}${LOCK_EXP}
DHCPD_LEASES=${TOP}/var/db/dhcpd.leases

HOSTS_ALLOW=${TOP}/etc/hosts.allow
HOSTS_ALLOW_LOCK=${HOSTS_ALLOW}${LOCK_EXP}

NTP_CONF=${TOP}/etc/ntp.conf
NTP_CONF_LOCK=${NTP_CONF}${LOCK_EXP}

CROND_FILE=${TOP}/etc/ibsbat/router
CROND_FILE_LOCK=${CROND_FILE}${LOCK_EXP}

ACCESS_CONF=${TOP}/usr/local/apache/conf/access.conf
ACCESS_CONF_LOCK=${ACCESS_CONF}${LOCK_EXP}

PUBLIC_HTML_TOP=${TOP}/home/ibsuser/public_html


NULL_CREATE_FILE="DHCPD_CONF_LOCK HOSTS_ALLOW_LOCK NTP_CONF_LOCK RC_CONF_LOCK CROND_FILE_LOCK"

DEBUG_NAME_LIST="wan_ifname lan_ifname DHCPD_CONF DHCPD_CONF_LOCK \
		HOSTS_ALLOW HOSTS_ALLOW_LOCK NTP_CONF NTP_CONF_LOCK RC_CONF RC_CONF_LOCK \
		CROND_FILE CROND_FILE_LOCK ACCESS_CONF"

. /etc/rc.subr

debug_display()
{
    for i in ${DEBUG_NAME_LIST}
    do
        eval _env=\$$i
	echo $_env
    done

    for i in ${DEBUG_NAME_LIST}
    do
        eval _tmp=\$$i
        __tmp=${_tmp%/*}
	echo $__tmp
    done
}

chkmyname()
{
    myname=`echo $USER`
    if [ "$myname" != "root" ]; then
        echo "$0 : User is not root!. This command execute at root."
	echo ""
	usage
	exit 1
    fi
    
}

chkargs()
{
    if [ "X$1" = "X" -a "X$2" = "X" ]; then
       #usage
       :
    elif [ "X$2" = "X" ]; then
       usage
    else
      lan_ifname="$1"
      wan_ifname="$2"
    fi
}

usage()
{
   echo "Usage : $0 [lan_name [wan_name]]"
   exit 0
}

null_create_file()
{
    for i in ${NULL_CREATE_FILE}
    do
        eval _nfile=\$$i
	eval __ndir=\$$i
	_ndir=${__ndir%/*}
	if [ ! -f $_nfile ]; then
	    # check directory
	    if [ "$DEBUG" = "1" ]; then
	        echo "mkdir -p $_ndir"
	        echo "cp /dev/null $_nfile"
	    else
	        if [ ! -d $_ndir ]; then
	            mkdir -p $_ndir
	        fi
	        cp /dev/null $_nfile
	    fi
	fi
    done
}

create_file()
{
    IFCONFIG_WAN=${TOP}/etc/ifconfig.${wan_ifname}
    IFCONFIG_WAN_LOCK=${IFCONFIG_WAN}.ibs-soho.lock

    IFCONFIG_LAN=${TOP}/etc/ifconfig.${lan_ifname}
    IFCONFIG_LAN_LOCK=${IFCONFIG_LAN}.ibs-soho.lock
}



set_ifname()
{
    for i in ${wan_ifname} ${lan_ifname}
    do
        if [ ! -f ${TOP}/etc/ifconfig.$i ]; then
           echo "File not found. [ ${TOP}/etc/ifconfig.$i ]"
	else
	   while read args; do
	        case "$args" in
	        "#"*|create)
			;;
		"!"*)
			eval ${args#*!}
			;;
		"inet"*)
			echo "$i:inet:: $args"
			;;
		*)
			echo "$i:$args"
			;;
		esac
	   done < ${TOP}/etc/ifconfig.$i
        fi
    done
}


chk_ntp_conf()
{
    _ntp_conf_f=${NTP_CONF}
    _flags=0
    if [ ! -f $_ntp_conf_f ]; then
cat >> $_ntp_conf_f << EOF
pidfile		/var/run/ntpd.pid
driftfile	/var/db/ntp.drift
logconfig	-syncstatus
#server		0.0.0.0		#ibs-soho
EOF
        return 0
    fi

    while read _server _add _key; do
        case "$_key" in
	"#ibs-soho")
		_flags=1
		;;
	*)
		;;
	esac
    done < $_ntp_conf_f

    if [ "$_flags" = "0" ];then
        echo "#server 0.0.0.0 #ibs-soho" >> $_ntp_conf_f
    fi
}

chk_exist_val()
{
    __key=$1
    if [ -z "$__key" ]; then
        return 1
    fi

    while read __args; do
        set -- $__args
	__top=$1
	#echo "__top :: $__top"
	while [ ! -z $1 ]; do
	    if [ "X$1" = "X#ibs-soho" ]; then
	        case "$__top" in
		"$__key"* | \#"$__key"*)
		  #echo "case :: $1"
		  return 0 ;;
		*) #echo "default case :: $1"
		  ;;
		esac
	    fi
	    shift 1
	done
    done < ${RC_CONF}
    return 1
}

chk_sticky()
{
    if [ ! -x /bin/chmod ]; then return 0; fi
    if [ ! -x /usr/bin/find ]; then return 0; fi
    find "${PUBLIC_HTML_ROOT}/Admin/Sys_setup/Router" -type f -name '*.cgi' -exec chmod 4755 {} \;
}

chk_rc_conf()
{

    if [ -z "${lan_interface_name}" ]; then
        echo "lan_interface_name=\"${lan_ifname}\"    #ibs-soho" >> ${RC_CONF}
    fi

    if [ -z "${wan_interface_name}" ]; then
        echo "wan_interface_name=\"${wan_ifname}\"    #ibs-soho" >> ${RC_CONF}
    fi


    if chk_exist_val dhcpd; then
        echo "dhcpd: exist"
    else
        echo "dhcpd: not exist"
    fi

    if chk_exist_val ntpd; then
        echo "ntpd: exist"
    else
        echo "ntpd: not exist"
    fi

    if chk_exist_val dhclient; then
        echo "ntpd: exist"
    else
        echo "ntpd: not exist"
    fi
}


####################
# option
####################
cfg_prev=
for cfg_option
do
  # If the previous option needs an argument, assign it.
  if test -n "$cfg_prev"; then
    eval "$cfg_prev=\$cfg_option"
    cfg_prev=
    continue
  fi

  case "$cfg_option" in
  -*=*) cfg_optarg=`echo "$cfg_option" | sed 's/[-_a-zA-Z0-9]*=//'` ;;
  *) cfg_optarg= ;;
  esac

#echo $cfg_prev
#echo $cfg_option
#echo $cfg_optarg

  case "$cfg_option" in
  --help | -help | -h)
    cat <<EOF
Usage: config_ibsrt.bat [options]
Options:
  --lan-ifname=IFNAME
EOF
    exit 0 ;;
  --lan-ifname)
    cfg_prev=lan_ifname ;;
  --lan-ifname=*)
    lan_ifname="$cfg_optarg" ;;
  --wan-ifname)
    cfg_prev=wan_ifname ;;
  --wan-ifname=*)
    lan_ifname="$cfg_optarg" ;;

  -*) { echo "config_ibs.bat: error: $cfg_option: invalid option;" 1>&2; exit 1; }
    ;;
  *)
    if test -n "`echo $cfg_option| sed 's/[-a-z0-9.]//g'`"; then
      echo "config_ibs.bat: warning: $cfg_option: invalid host type" 1>&2
    fi
    ;;

  esac
done

echo $lan_ifname
echo $wan_ifname

exit 0



##################################################

. ${RC_CONF}

chkmyname
chkargs
#set_ifname
null_create_file
#create_file

debug_display

chk_rc_conf


#echo "dhcpd: ${dhcpd}"
#echo "dhcpd_flags: ${dhcpd_flags}"

#. ./test.conf
#echo "aaaaa: $aaaaa"
#echo "bbbbb: $bbbbb"
#echo "ccccc: $ccccc"

