#!/bin/sh
#
# DNS

[ "X${X7DBG}" = "Xon" ] && set -x

CMDBAT=`basename $0`
WEBTOP=/etc/ibsbat/Router
WEB_DNS_TXT=${WEBTOP}/dns/dns.txt
WEB_DNS_TABLE_TXT=${WEBTOP}/dns/dns_table.txt
WEB_HOST_TABLE_TXT=${WEBTOP}/dns/host_table.txt

DNSTOP=/etc/ibsbat/dns
DNSTBLAWK=${DNSTOP}/dns_table.awk
DNSTBLCONF=${DNSTOP}/dns_table.conf
HOSTTBLAWK=${DNSTOP}/host_table.awk
HOSTTBLCONF=${DNSTOP}/host_table.conf

dns_syslog()
{
	logger -p kern.info -t $CMDBAT $1
}

#
# $1: ipaddr
#
acl_set()
{
	if [ -z "$1" ]; then
		return 0
	fi

	if [ -f /etc/named.conf ]; then
		_nconf=/etc/named.conf
	else
		_nconf=/etc/ibsbat/namedb/named.conf
	fi

	cat $_nconf | \
		awk -v acl_ip=$1 '/.*#ibs-acl-lan/{printf"  %s;  #ibs-acl-lan\n",acl_ip}!/.*#ibs-acl-lan/{print}' \
		> /etc/named.conf.1
	mv /etc/named.conf.1 /etc/named.conf
	if [ -x /etc/rc.d/init.d/named ]; then
		/etc/rc.d/init.d/named restart
	fi
	return 0
}

create_dns_table()
{
	cat ${WEB_DNS_TABLE_TXT} | awk -f ${DNSTBLAWK} > ${DNSTBLCONF}
}

create_host_table()
{
	cat ${WEB_HOST_TABLE_TXT} | awk -f ${HOSTTBLAWK} > ${HOSTTBLCONF}
}

#
# $1: ifno (real)
#
dns_route_add_ifno()
{
	_pppif=$1
	
}

dns_route_add()
{
	_domain=$1
	_dnsip=$2

	if [ -z "$_domain" -o -z "$_dnsip" ]; then
		return 1
	fi

}

resolv_conf_local()
{
	if [ -f /etc/ibsbat/namedb/resolv.conf ]; then
		cp -f /etc/ibsbat/namedb/resolv.conf.org /etc/resolv.conf
	else
		echo "nameserver 127.0.0.1" > /etc/resolv.conf
	fi
}

dns_default()
{
	if [ -f "$WEB_DNS_TXT" ]; then
		. ${WEB_DNS_TXT}
	else
		return 0
	fi

	if [ "X${refer_dns}" = "Xmanual" ]; then
		if [ -f /etc/named.conf ]; then
			_nconf=/etc/named.conf
		else
			_nconf=/etc/ibsbat/namedb/named.conf
		fi

		if [ -z "$primary_dns" -a -z "$secondary_dns" ]; then
			dns_syslog "pri and sec not set. refer_dns:$refer_dns"
			return 0
		fi

		_forwarders="${primary_dns} ;"
		_forwarders="${_forwarders} ${secondary_dns} ;"
		cat $_nconf| \
			awk -v fwds="$_forwarders" '/.*#ibs-forward/{printf"  %s  #ibs-forward\n",fwds}!/.*#ibs-forward/{print}' \
			> /etc/named.conf.1
		mv /etc/named.conf.1 /etc/named.conf
		if [ -x /etc/rc.d/init.d/named ]; then
			/etc/rc.d/init.d/named restart
		fi
	fi
	return 0
}

#
# $1: add/del
# $2: ifname ( ppp0/ppp1/... )
# $3: realif ( ppp0/ppp1/... )
#
dns_zone_ppp()
{
	_spec=$1
	_pppentry=${2#ppp}
	_realif=$3
	if [ -z "$_pppentry" -o -z "$_realif" ]; then
		return 1
	fi
	
	[ -f ${DNSTBLCONF} ] && . ${DNSTBLCONF}

	_entno=0
	while [ ${_entno} -le 15 ]; do
		_entry=ENTRY_DNS_${_entno}
		_dns_appoint=`eval echo \$\{${_entry}_DNS_APPOINT\}`

		if [ "X${_dns_appoint}" = "X$_pppentry" ]; then
			_dns_domain=`eval echo \$\{${_entry}_DNS_DOMAIN\}`
			_dns_server=`eval echo \$\{${_entry}_DNS_SERVER\}`

			dns_syslog "dns ppp: $_entno:$_dns_domain:$_dns_appoint"
			if [ -z "$_dns_domain" -o -z "$_dns_appoint" ] ; then
				_entno=`expr ${_entno} + 1`
				continue
			fi

			/bin/sh /etc/ibsbat/dns/dnslookup2.bat $_spec ${_realif} $_dns_domain
		fi

		_entno=`expr ${_entno} + 1`
	done
}

#
#
dns_zone_fix()
{
	_spec=$1

	[ -f ${DNSTBLCONF} ] && . ${DNSTBLCONF}

	_entno=0
	while [ ${_entno} -le 15 ]; do
		_entry=ENTRY_DNS_${_entno}

		_dns_appoint=`eval echo \$\{${_entry}_DNS_APPOINT\}`

		if [ "X${_dns_appoint}" = "Xip" ]; then
			_dns_domain=`eval echo \$\{${_entry}_DNS_DOMAIN\}`
			_dns_server=`eval echo \$\{${_entry}_DNS_SERVER\}`

			dns_syslog "dns fix: $_entno:$_dns_domain:$_dns_server"
			if [ -z "$_dns_domain" -o -z "$_dns_server" ] ; then
				_entno=`expr ${_entno} + 1`
				continue
			fi

			/bin/sh /etc/ibsbat/dns/dnslookup2.bat $_spec $_dns_server $_dns_domain
		fi

		_entno=`expr ${_entno} + 1`
	done
}

#
#ۥϿ
#
#zone "aaaa" {
#	type master;
#	file "s/hostA";
#	allow-query { x7000net; };
#};
#
#zone "1.26.172.in-addr.arpa" {
#	type master;
#	file "s/entry1";
#	allow-query { x7000net; };
#};
host_zone_fix_head()
{
	_sfile=$1
	echo "\$TTL 3600" >> $_sfile
	echo "@   IN   SOA   x7000.ibs. root.x7000.ibs. (" >> $_sfile
	echo "               2004040101 ; "                >> $_sfile
	echo "               3600       ; "                >> $_sfile
	echo "               300        ; "                >> $_sfile
	echo "               3600000    ; "                >> $_sfile
	echo "               3600 )     ; "                >> $_sfile
	echo "    IN    NS   localhost."                   >> $_sfile
}
host_zone_fix()
{
	_spec=$1

	_nsupdate=/tmp/nsupdate.txt

	[ -f $_nsupdate ] && rm -f $_nsupdate
	for i in /tmp/dns_zone_* ; do
		[ -f ${i} ] && rm -f ${i}
	done


	[ -f ${HOSTTBLCONF} ] && . ${HOSTTBLCONF}

	host_zone_fix_head /var/namedb/named.host

	_entno=0
	while [ ${_entno} -le 63 ]; do
		_entry=ENTRY_HOST_${_entno}

		_host_name=`eval echo \$\{${_entry}_HOST_TABLE_HOST\}`
		_host_ip=`eval echo \$\{${_entry}_HOST_TABLE_IP\}`

		_dot=`echo $_host_name | awk '{printf"%d",index($0,".");}'`

		if [ "X$_dot" != "X0" ]; then
			_hostname=${_host_name%%.*}
			_domainname=${_host_name#*.}
			_ipentry=${_host_ip%%.*}
			_ipzone=${_host_ip#*.}

			_zone_file=/tmp/dns_zone_${_domainname}
			if [ ! -f "$_zone_file" ]; then
				echo "zone \"$_domainname\" {   //ibs-soho-host-entry" >> "$_zone_file"
				echo "   type master;        //ibs-soho-host-entry" >> "$_zone_file"
				echo "   file \"${_domainname}\";   //ibs-soho-host-entry" >> "$_zone_file"
				echo "   allow-update { localhost; };    //ibs-soho-host-entry" >> "$_zone_file"
				echo "};                 //ibs-soho-host-entry" >> "$_zone_file"
			fi

			_zone_file1=/var/namedb/${_domainname}
			if [ ! -f "$_zone_file1" ]; then
				host_zone_fix_head $_zone_file1
			fi

			echo "update add $_host_name 3600 IN A $_host_ip" >> $_nsupdate

		fi


		_entno=`expr ${_entno} + 1`
	done

	echo " " >> $_nsupdate

	cp /etc/named.conf /etc/named.conf.1
	for i in /tmp/dns_zone_* ; do
		[ -f ${i} ] && cat ${i} >> /etc/named.conf.1
	done
	mv /etc/named.conf.1 /etc/named.conf
	/etc/rc.d/init.d/named restart

	/usr/bin/nsupdate ${_nsupdate} > /dev/null 2>&1
}
host_zone_fix_dummy()
{
	_spec=$1

	[ -f ${HOSTTBLCONF} ] && . ${HOSTTBLCONF}

	host_zone_fix_head /var/namedb/named.host

	_entno=0
	while [ ${_entno} -le 63 ]; do
		_entry=ENTRY_HOST_${_entno}

		_host_name=`eval echo \$\{${_entry}_HOST_TABLE_HOST\}`
		_host_ip=`eval echo \$\{${_entry}_HOST_TABLE_IP\}`

		_dot=`echo $_host_name | awk '{printf"%d",index($0,".");}'`

		if [ "X$_dot" != "X0" ]; then
			_hostname=${_host_name%%.*}
			_domainname=${_host_name#*.}
			_ipentry=${_host_ip%%.*}
			_ipzone=${_host_ip#*.}

			_zone_file=/tmp/dns_zone_${_domainmame}
			if [ ! -f "$_zone_file" ]; then
				echo "zone \"$_domainname\" {   //ibs-soho-host-entry" >> "$_zone_file"
				echo "   type master;        //ibs-soho-host-entry" >> "$_zone_file"
				echo "   file \"named.host\";   //ibs-soho-host-entry" >> "$_zone_file"
				echo "};                 //ibs-soho-host-entry" >> "$_zone_file"
			fi

			_ip_file=/tmp/dns_ip_${_ipzone}
			if [ ! -f "$_ip_file" ]; then
				echo "zone \"${_ipzone}.in-addr.arpa\" {   //ibs-soho-host-entry" >> "$_ip_file"
				echo "   type master;        //ibs-soho-host-entry" >> "$_ip_file"
				echo "   file \"${_ipzone}.rev\";   //ibs-soho-host-entry" >> "$_ip_file"
				echo "};                 //ibs-soho-host-entry" >> "$_ip_file"
			fi

			_zone_file1=/var/namedb/named.host
			echo "$_host_name	   IN	A	$_host_ip" >> $_zone_file1

			_ipzone_file1=/var/namedb/${_ipzone}.rev
			if [ ! -f "$_ipzone_file1" ]; then
				host_zone_fix_head $_ipzone_file1
			fi
			echo "$_ipentry	   IN	PTR	${_host_name}." >> $_ipzone_file1
		fi

		_entno=`expr ${_entno} + 1`
	done

	cp /etc/named.conf /etc/named.conf.1
	for i in /tmp/dns_zone_* ; do
		[ -f ${i} ] && cat ${i} >> /etc/named.conf.1
	done
	for i in /tmp/dns_ip_* ; do
		[ -f ${i} ] && cat ${i} >> /etc/named.conf.1
	done
}

####################
_spec=$1
case $_spec in
	boot)
		create_dns_table
		create_host_table
		dns_default
		dns_zone_fix add
		host_zone_fix
		;;
	default)
		dns_default
		;;
	acl)
		acl_set $2
		;;
	addzoneppp)
		dns_zone_ppp add $2 $3
		;;
	addzonefix)
		dns_zone_fix add
		;;
	delzoneppp)
		dns_zone_ppp del $2 $3
		;;
	delzonefix)
		dns_zone_fix del
		;;
	host_zone)
		create_host_table
		host_zone_fix
		;;
	*)
		create_dns_table
		create_host_table
		;;
esac

exit 0
