#!/bin/sh
#
# $1: spec(add/del)
# $2: real ifname (ppp0/ppp1...  or IP) 
# $3: zone name
#
CMDBAT=`basename $0`
_lookup_f=/tmp/lookup
_lookup_e=/tmp/lookup.err
_output_f=/etc/named.conf.lookup
_dnslookup_key_prefix="#ibs-soho-dnslookup-"

dnslookup_syslog()
{
  logger -p kern.info -t $CMDBAT "$1"
}

output_zone()
{
  _entry=$1
  _zone=$2
  _servers=$3

  echo "" > ${_output_f}
  echo "zone \"$_zone\" {    ${_dnslookup_key_prefix}${_zone}${_entry}" >> ${_output_f}
  echo "  type forward;      ${_dnslookup_key_prefix}${_zone}${_entry}" >> ${_output_f}
  echo "  forward only;      ${_dnslookup_key_prefix}${_zone}${_entry}" >> ${_output_f}
  echo "  forwarders { ${_servers} }; ${_dnslookup_key_prefix}${_zone}${_entry}" >> ${_output_f}
  echo "};                  ${_dnslookup_key_prefix}${_zone}${_entry}" >> ${_output_f}
  
  cat ${_output_f} >> /etc/named.conf
  /etc/rc.d/init.d/named restart
}


_spec=$1
_realif=$2
_zone=$3

RETVAL=0

if [ -z "$_spec" -o -z "$_realif"  -o -z "$_zone" ]; then
  exit 1
fi

case $_realif in
ppp*)
  if [ -f /etc/ppp/resolv_${_realif}.conf ]; then
    _list=`cat /etc/ppp/resolv_${_realif}.conf | awk '{printf"%s ",$2}'`
  else
    exit 1
  fi
  ;;
*)
  _list="$_realif"
  _realif=fix
  ;;
esac


case $_spec in
add)
  if [ -z "$_list" ]; then
    exit 1
  fi

  _srch_key="${_dnslookup_key_prefix}${_zone}${_realif}"
  grep "${_srch_key}" /etc/named.conf > /dev/null 2>&1
  if [ $? = 0 ]; then
    dnslookup_syslog "Already exist entry: $_realif:$_zone"
    exit 1
  fi

  for i in $_list
  do
#    nslookup -query=NS $_zone $i > $_lookup_f 2>$_lookup_e
#    if [ -s $_lookup_e ]; then
#      dnslookup_syslog "$_lookup_e is not 0"
#      RETVAL=1
#      continue
#    else
#      RETVAL=0
#    fi
#
#    if [ ${RETVAL} = 0 ]; then
#      _server=`cat $_lookup_f | awk '/^Address:/{printf"%s ",$2}'`
#      _server_list="$_server_list $_server;"
#    fi
     _server_list="$_server_list $i ;"
  done

  if [ "${RETVAL}" = 0 ]; then
    output_zone ${_realif} ${_zone} "${_server_list}"
    dnslookup_syslog "add list ${_realif} ${_server_list}"
    RETVAL=0
  fi
  ;;
del)
  if [ -f /etc/named.conf ]; then
    cat /etc/named.conf |grep -v "${_dnslookup_key_prefix}${_zone}${_realif}" > /etc/named.conf.1
    mv /etc/named.conf.1 /etc/named.conf
    /etc/rc.d/init.d/named restart
    dnslookup_syslog "del list ${_realif}"
  fi
  RETVAL=0
  ;;
*)
  ;;
esac
exit $RETVAL
