#!/bin/sh
#
#  For SCAS Filter
#
[ "X${X7DBG}" = "Xon" ] && set -x

CMDBAT=`basename $0`
IPTABLES=/sbin/iptables

ADD_RC=/tmp/addcon.result
ENTFILE=/etc/ibsbat/Router/filtering/filtering_table.txt
AWKFILE=/etc/ibsbat/filter/ftbl_scas.awk
FNCFILE=/tmp/ftbl_entry_scas

VOIP_DATA=/tmp/voip_data

CONT_FILE=/etc/ibsbat/Router/filtering/filtering_set_wan.txt
CONT_AWK=/etc/ibsbat/filter/ftblwan_scas.awk
CONT_RESULT=/tmp/ftblwan_scas.txt

WANIF=eth0

INIT_MAXCOUNT=128

[ -f "$VOIP_DATA" ] && . $VOIP_DATA
[ -n "$my_port" ]   || my_port=5060
[ -n "$start_rtp" ] || start_rtp=5100
[ -n "$end_rtp"   ] || end_rtp=5140

# 2006.05.13
[ "X$my_port" = "X0" ] && my_port=5060
[ "X$start_rtp" = "X0" ] && start_rtp=5100
[ "X$end_rtp" = "X0" ] && end_rtp=5140

scfil_syslog()
{
	logger -p kern.info -t $CMDBAT "$1"
}

create_scas_entry()
{
	[ -f "${AWKFILE}" ] || ( scfil_syslog; return 1)

	cat ${ENTFILE}   | awk -f ${AWKFILE} > ${FNCFILE}
	cat ${CONT_FILE} | awk -f ${CONT_AWK} > ${CONT_RESULT}
}


filter_init(){
	${IPTABLES} -N scas

	local count=${INIT_MAXCOUNT}
	while [ $count -gt 0 ]; do
		${IPTABLES} -D INPUT -i $1 -j scas
		if [ $? != 0 ]; then break; fi
		count=$(( $count - 1 ))
	done

	local count=${INIT_MAXCOUNT}
	while [ $count -gt 0 ]; do
		${IPTABLES} -D OUTPUT -o $1 -j scas
		if [ $? != 0 ]; then break; fi
		count=$(( $count - 1 ))
	done

	local count=${INIT_MAXCOUNT}
	while [ $count -gt 0 ]; do
		${IPTABLES} -D FORWARD -i $1 -j scas
		if [ $? != 0 ]; then break; fi
		count=$(( $count - 1 ))
	done

	local count=${INIT_MAXCOUNT}
	while [ $count -gt 0 ]; do
		${IPTABLES} -D FORWARD -o $1 -j scas
		if [ $? != 0 ]; then break; fi
		count=$(( $count - 1 ))
	done

	${IPTABLES} -A INPUT -i $1 -j scas
	${IPTABLES} -A OUTPUT -o $1 -j scas
	${IPTABLES} -A FORWARD -i $1 -j scas
	${IPTABLES} -A FORWARD -o $1 -j scas
}

filter_flush(){

	${IPTABLES} -t filter -F scas
	${IPTABLES} -t nat -F scas
	${IPTABLES} -t mangle -F scas
}

### SIP
# $1: i/f
# $2: sipport
# $3: my sipport
f_no_100_scas()
{
	${IPTABLES} -I scas -o $1 -p udp --dport $2 --sport $3 -j ACCEPT
	if [ $? = 0 ]; then
		scfil_syslog "scas filter set.(SIP OUT ACCEPT)  dst:$2 and src:$3"
	else
		scfil_syslog "scas filter set error. (SIP OUT 1:$1  2:$2  3:$3)"
	fi
}

### RTP
# $1: i/f
# $2: rtpport1
# $3: rtpport2
f_no_101_scas()
{
	if [ -n "$2" -a -n "$3" ]; then
		${IPTABLES} -I scas -o $1 -p udp --sport ${2}:${3} -j ACCEPT
		if [ $? = 0 ]; then
			scfil_syslog "scas filter set.(RTP OUT ACCEPT)  src:${2} - ${3}"
		else
			scfil_syslog "scas filter set error. (RTP OUT 1:$1  2:$2  3:$3)"
		fi
	else
		echo "arg2 or arg3 is null"
	fi
}

### SIP
# $1: i/f
# $2: sipport
# $3: my sipport
f_no_102_scas()
{
	${IPTABLES} -I scas -i $1 -p udp --sport $2 --dport $3  -j ACCEPT 
	if [ $? = 0 ]; then
		scfil_syslog "scas filter set.(SIP IN ACCEPT)  src:$2 and dst:$3"
	else
		scfil_syslog "scas filter set error. (SIP IN 1:$1  2:$2  3:$3)"
	fi
}

### RTP
# $1: i/f
# $2: rtpport1
# $3: rtpport2
f_no_103_scas()
{
	if [ -n "$2" -a -n "$3" ]; then
		${IPTABLES} -I scas -i $1 -p udp --dport ${2}:${3} -j ACCEPT
		if [ $? = 0 ]; then
			scfil_syslog "scas filter set.(RTP IN ACCEPT)  dst:${2} - ${3}"
		else
			scfil_syslog "scas filter set error. (RTP IN 1:$1  2:$2  3:$3)"
		fi
	else
		echo "arg2 or arg3 is null"
	fi
}

# $1: entry_no
check_on_off()
{
	[ -f "${CONT_RESULT}" ] || return 1

	eval `grep filtering_set_wan_${1} ${CONT_RESULT}`
	check=`eval echo \$\{filtering_set_wan_${1}\}`

	if [ -n "${check}" -a "${check}" = "on" ]; then
		return 0
	else
		return 1
	fi
}

## 2006.05.04
f_no_100_scas_force()
{
	if [ -f ${ADD_RC} ]; then
		eval `grep sipsv_port ${ADD_RC}`
		scfil_syslog "sipsv_port:${sipsv_port}"
		[ -n "${sipsv_port}" ] || sipsv_port=5060

		eval `grep regsv_port ${ADD_RC}`
		scfil_syslog "regsv_port:${regsv_port}"
		[ -n "${regsv_port}" ] || regsv_port=5060
	else
		sipsv_port=5060
		regsv_port=5060
	fi
	f_no_100_scas ${WANIF} ${sipsv_port} ${my_port}
	f_no_100_scas ${WANIF} ${regsv_port} ${my_port}
	return 0
}
f_no_101_scas_force()
{
	if [ -f ${ADD_RC} ]; then
		eval `grep rtpport1 ${ADD_RC}`
		scfil_syslog "rtpport1:${rtpport1}"
		eval `grep rtpport2 ${ADD_RC}`
		scfil_syslog "rtpport2:${rtpport2}"

		if [ ! -n "${rtpport1}" -o ! -n "${rtpport2}" ]; then
			rtpport1=$start_rtp
			rtpport2=$end_rtp
		fi
	else
		rtpport1=$start_rtp
		rtpport2=$end_rtp
	fi
	f_no_101_scas ${WANIF} ${rtpport1} ${rtpport2}
	return 0
}
f_no_102_scas_force()
{
	if [ -f ${ADD_RC} ]; then
		eval `grep sipsv_port ${ADD_RC}`
		scfil_syslog "sipsv_port:${sipsv_port}"
		[ -n "${sipsv_port}" ] || sipsv_port=5060

		eval `grep regsv_port ${ADD_RC}`
		scfil_syslog "regsv_port:${regsv_port}"
		[ -n "${regsv_port}" ] || regsv_port=5060
	else
		sipsv_port=5060
		regsv_port=5060
	fi
	f_no_102_scas ${WANIF} ${sipsv_port} ${my_port}
	f_no_102_scas ${WANIF} ${regsv_port} ${my_port}
	return 0
}
f_no_103_scas_force()
{
	if [ -f ${ADD_RC} ]; then
		eval `grep rtpport1 ${ADD_RC}`
		scfil_syslog "rtpport1:${rtpport1}"
		eval `grep rtpport2 ${ADD_RC}`
		scfil_syslog "rtpport2:${rtpport2}"

		if [ ! -n "${rtpport1}" -o ! -n "${rtpport2}" ]; then
			rtpport1=$start_rtp
			rtpport2=$end_rtp
		fi
	else
		rtpport1=$start_rtp
		rtpport2=$end_rtp
	fi
	f_no_103_scas ${WANIF} ${rtpport1} ${rtpport2}
	return 0
}
# $1: i/f
# $2: port
f_no_update_cas_force()
{
	${IPTABLES} -I scas -i $1 -p tcp --sport $2 -j ACCEPT
	if [ $? = 0 ]; then
		scfil_syslog "scas filter set.(UPDATE_CAS IN ACCEPT)  src:$2"
	else
		scfil_syslog "scas filter set error. (UPDATE_CAS IN 1:$1  2:$2)"
	fi
	${IPTABLES} -I scas -i $1 -p udp --sport $2 -j ACCEPT

	${IPTABLES} -I scas -o $1 -p tcp --dport $2 -j ACCEPT
	if [ $? = 0 ]; then
		scfil_syslog "scas filter set.(UPDATE_CAS IN ACCEPT)  dst:$2"
	else
		scfil_syslog "scas filter set error. (UPDATE_CAS IN 1:$1  2:$2)"
	fi
	${IPTABLES} -I scas -o $1 -p tcp --dport $2 -j ACCEPT
}
f_no_update_cas()
{
	local upd_port=
	if [ -f ${ADD_RC} ]; then
		eval `grep url_update_cas ${ADD_RC}`
		scfil_syslog "${url_update_cas}"

		if [ -n "${url_update_cas}" ]; then
			local tmp1=${url_update_cas#http://}
			local tmp2=${tmp1%%\/*}
			local upd_port=${tmp2##*:}
		fi
		if [ -n "${upd_port}" ]; then
			f_no_update_cas_force ${WANIF} ${upd_port}
		fi
	fi

	return 0
}

##############################
create_scas_entry
filter_init eth0
filter_flush

if [ -f "${FNCFILE}" ]; then
	. ${FNCFILE}
	_flist=`grep "f_no_[0-9][0-9][0-9]" ${FNCFILE} | sed -e 's/(//' -e 's/)//'`
else
	echo "File not found: ${FNCFILE}"
	exit 1
fi

f100_done=1
f101_done=1
f102_done=1
f103_done=1

for i in $_flist; do
	echo "FUNC:${i}"
	if [ "${i}" = "f_no_100" ]; then
		f_no_100_scas_force
		if [ $? = 0 ]; then
			f100_done=0
		fi
		continue
	fi
	if [ "${i}" = "f_no_101" ]; then
		f_no_101_scas_force
		if [ $? = 0 ]; then
			f101_done=0
		fi
		continue
	fi

	if [ "${i}" = "f_no_102" ]; then
		f_no_102_scas_force
		if [ $? = 0 ]; then
			f102_done=0
		fi
		continue
	fi
	if [ "${i}" = "f_no_103" ]; then
		f_no_103_scas_force
		if [ $? = 0 ]; then
			f103_done=0
		fi
		continue
	fi

	check_on_off ${i##f_no_}
	if [ $? = 0 ]; then
		${i} ${WANIF}
	fi
done

[ "X$f100_done" != "X0" ] && f_no_100_scas_force
[ "X$f101_done" != "X0" ] && f_no_101_scas_force
[ "X$f102_done" != "X0" ] && f_no_102_scas_force
[ "X$f103_done" != "X0" ] && f_no_103_scas_force

f_no_update_cas
