#!/bin/sh
#
# ftpupload.bat
# args:
#   $1: ftp server address
#   $2: user name
#   $3: password
#   $4: type
#   $5: remote file
#   $6: debug
# 
# 2004.03.09
#

CMDBAT=`basename $0`
LOCKFILE=/var/lock/${CMDBAT}.lock
FTPUPDEBUG=

DEBUGLOG=/tmp/ftpupload.txt

. /etc/ibsbat/ibs.subr
#. /etc/ibsbat/ibsflash.conf

_remote_file=""
RETVAL=0


#
# $1: on/off
ftpupdate_state()
{
	if [ -z "$1" ]; then
		return 0
	fi
	case $1 in
		on)
			ibs_lock $0 ${FTPUPDATE_STATE_ON} 20
			if [ $? != 0 ]; then
				ftpupload_syslog "Other process flash writing. update abort."
				return 1
			else
				return 0
			fi
			;;
		off)
			ibs_unlock ${FTPUPDATE_STATE_ON}
			if [ $? != 0 ]; then
				ftpupload_syslog "Unlock error"
				return 1
			else
				return 0
			fi
			;;
		*)
			;;
	esac
}

ftpupload_syslog()
{
	logger -p kern.info -t $CMDBAT $1
}

ftpup_led_update()
{
	LED_PROC_LIST="/proc/update /proc/ibsled/update /proc/ibs/ibsled/update"
	_led_type=$1

	if [ -z "$_led_type" ]; then
		_led_type=0
	fi

  # LED on : voip
	for led in ${LED_PROC_LIST}; do
		if [ -f "${led}" ]; then
			echo "$_led_type" > "${led}"
		fi
	done
}

#
# $1 : remote file name
# $2 : local  file name
get_ftp()
{
	_getfile=$1
	_locfile=$2

	if [ -x /usr/bin/wget ]; then
		if [ -z "$_locfile" ]; then
			WGETOPT=
		else
			WGETOPT="-O $_locfile"
		fi
		wget -nv $WGETOPT ftp://$_user:$_passwd@$_serverip/$_getfile >>$DEBUGLOG 2>&1
		RETVAL=$?
	else

		_dir=${_getfile%/*}
		echo "1:_getfile:${_getfile} _dir:${_dir}" >>$DEBUGLOG
		if [ "X${_getfile}" = "X${_dir}" ]; then
			_dir="."
		else
			_getfile=${_getfile##*/}
		fi
		echo "2:_getfile:${_getfile} _dir:${_dir}" >>$DEBUGLOG

		cat <<EOF  | ftp -n
open $_serverip
user $_user $_passwd
bin
cd $_dir
get $_getfile $_locfile
close
EOF

if [ -f "$_locfile" ]; then
	RETVAL=0
else
	RETVAL=1
fi
	fi

	return $RETVAL
}

# IBS$B%"%W%j(B
up_ibsexe()
{
	STRINGS="IBS APP execute"
	_local_file=ibsexe.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi

	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_ibsexe.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		tar xzvpf $_local_file -C /tmp3 2>&1 >>$DEBUGLOG
		echo "Copying ${STRINGS} to ${TAR_IBSEXE}"
		echo "Please wait..."
		cd /tmp3
		ibsexe_umount
		RETVAL=$?
		if [ ${RETVAL} = 0 ]; then
			find exe -type f -exec ls -l {} \; > ibsexe_ver.txt
			mv ibsexe_ver.txt exe
			mkcramfs exe ${TAR_IBSEXE} >>$DEBUGLOG 2>&1
			ibsexe_mount
		else
			echo "umount error" >>$DEBUGLOG
			find exe -type f -exec ls -l {} \; > ibsexe_ver.txt
			mv ibsexe_ver.txt exe
			mkcramfs exe ${TAR_IBSEXE} >>$DEBUGLOG 2>&1
			ibsexe_mount
		fi
		rm -fr exe
		cd /tmp2
		rm -f $_local_file
	fi
	return $RETVAL
}

up_router()
{
	RETVAL=0
	STRINGS="IBS ROUTER"
	_local_file=etc.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_etc.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		cat $_local_file|tar tzpf - |grep "etc/ibsbat" >/dev/null 2>&1
		if [ "$?" != "0" ]; then
			echo "Not router file!!"
			RETVAL=1
		else
			echo "Copying ${STRINGS} to ${TAR_ETC}"
			echo "Please wait..."
			echo "cp $_local_file ${TAR_ETC}" >>$DEBUGLOG
			cp $_local_file ${TAR_ETC}
			RETVAL=0
		fi
		rm -f $_local_file
	fi
	return $RETVAL
}

up_web()
{
	RETVAL=0
	STRINGS="IBS BROWSER"
	_local_file=home.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_home.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		cat $_local_file | tar tzpf - |grep "home/ibsuser/public_html" >/dev/null 2>&1
		if [ "$?" != "0" ]; then
			echo "Not browser file!!"
			RETVAL=1
		else
			echo "Copying ${STRINGS} to ${TAR_HOME}"
			echo "Please wait..."
			echo "cp $_local_file  ${TAR_HOME}" >>$DEBUGLOG
			cp $_local_file  ${TAR_HOME}
			md5sum ${TAR_HOME} > ${MD5_HOME}
			cp $_local_file  ${TAR_HOME_BAK}
			md5sum ${TAR_HOME_BAK} > ${MD5_HOME_BAK}
			RETVAL=0
		fi
		rm -f $_local_file
	fi
	return $RETVAL
}

up_ipkt()
{
#  if [ ! -z "${TAR_IPKT_COMP}" -a -f "${TAR_IPKT}" ]; then
#    rm -f ${TAR_IPKT}
#    up_ipkt_comp
#    RETVAL=$?
#    return $RETVAL
#  fi

	STRINGS="IBS IPKT"
	_local_file=ipkt.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_ipkt.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi
		echo "Copying ${STRINGS} to ${TAR_IPKT}"
		echo "Please wait..."
		echo "cp $_local_file  ${TAR_IPKT}" >>$DEBUGLOG
		cp $_local_file  ${TAR_IPKT}
		rm -f $_local_file
	fi
}

# IPKT cramfs
up_ipkt_comp()
{
	STRINGS="IBS IPKT"
	_local_file=s_ipkt_rom.s

	if [ -f "${TAR_IPKT}" ]; then
		rm -f ${TAR_IPKT}
	fi

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi

	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_ipkt.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		mkdir /tmp3/ibskt
		cp $_local_file /tmp3/ibskt 2>&1 >>$DEBUGLOG
		echo "Copying ${STRINGS} to ${TAR_IPKT_COMP}"
		echo "Please wait..."
		cd /tmp3
		if [ -f "${TAR_IPKT_COMP}" ]; then
			ipkt_umount
		fi
		RETVAL=$?
		if [ ${RETVAL} = 0 ]; then
			mkcramfs ibskt ${TAR_IPKT_COMP} >>$DEBUGLOG 2>&1
			ipkt_mount
		else
			echo "umount error" >>$DEBUGLOG
			mkcramfs ibskt ${TAR_IPKT_COMP} >>$DEBUGLOG 2>&1
			ipkt_mount
		fi
		rm -fr ibskt
		cd /tmp2
		rm -f $_local_file
	fi
	return $RETVAL
}

up_ibsgw()
{
	STRINGS="IBS GW"
	_local_file=ibsgw.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_ibsgw.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		echo "Copying ${STRINGS} to ${TAR_IBSGW}"
		echo "Please wait..."
		echo "cp $_local_file  ${TAR_IBSGW}" >>$DEBUGLOG
		cp $_local_file  ${TAR_IBSGW}
		rm -f $_local_file
	fi
}

up_pgdata()
{
	STRINGS="IBS PG DATA"
	_local_file=pgsql_data.tar.gz

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_pgsql_data.tar.gz
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		echo "Copying ${STRINGS} to ${TAR_PGSQL}"
		echo "Please wait..."
		echo "cp $_local_file  ${TAR_PGSQL}" >>$DEBUGLOG
		cp $_local_file  ${TAR_PGSQL}
		md5sum ${TAR_PGSQL} > ${MD5_PGSQL}
		touch /tmp/pgsql_nosave
		rm -f $_local_file
	fi
}

##########
#$B%7%9%F%`4XO"(B
up_system_kernel()
{
	_infile=$1
	echo "dd if=$_infile of=/dev/mtdblockkernel" >>$DEBUGLOG
	if [ -z "$FTPUPDEBUG" ]; then
		dd if=$_infile of=/dev/mtdblockkernel >/dev/null 2>&1
	fi
	rm -f $_infile
}
up_system_ramdisk()
{
	_infile=$1
	echo "dd if=$_infile of=/dev/mtdblockramdisk" >>$DEBUGLOG
	if [ -z "$FTPUPDEBUG" ]; then
		dd if=$_infile of=/dev/mtdblockramdisk >/dev/null 2>&1
	fi
	rm -f $_infile
}
up_system_ipl()
{
	_infile=$1
	echo "dd if=$_infile of=/dev/mtdblockipl" >>$DEBUGLOG
	if [ -z "$FTPUPDEBUG" ]; then
		dd if=$_infile of=/dev/mtdblockipl >/dev/null 2>&1
	fi
	rm -f $_infile
}
up_system_etc()
{
	_infile=$1
	echo "cp $_infile ${TAR_ETC}" >>$DEBUGLOG
	if [ -z "$FTPUPDEBUG" ]; then
		cp $_infile ${TAR_ETC}
	fi
	rm -f $_infile
}
up_system_usr()
{
	_infile=$1
	if [ -f /etc/ibsbat/update.txt ]; then
		cat /etc/ibsbat/update.txt
	else
		echo "System area updating..."
		echo "In this update, it writes in FLASH area and a System is rebooted automatically."
		echo "From several minutes, it takes about a maximum of 30 minutes."
		echo "For a while, please wait."
	fi
	touch /tmp/autoupdate
	if [ -z "$FTPUPDEBUG" ]; then
		init 1
	fi
}

up_system()
{
	STRINGS="IBS KERNEL"
	_local_file=kern.tar.gz
	_list_file=updatelist.txt
	_kern_file=vmlinux8241_ram.image
	_ram_file=ramdisk8241.image
	_ipl_file=ppcboot200_128M.image
	_etc_file=etc.tar.gz
	_usr_file=usr.cramfs
	_bat_file=ibssysupdate.bat

	if [ "X$_remote_file" = "X" ]; then
		_remote_file=$_local_file
	fi
	
	echo "$_remote_file $_local_file" >>$DEBUGLOG
	
	if [ "X$_debug" = "Xdebug" ]; then
		touch /tmp/${_debugf}_to_sytem
	else
		echo "Get ${STRINGS} file from FTP server"
		get_ftp $_remote_file $_local_file

		if [ "X$RETVAL" != "X0" ]; then
			echo "File not found."
			return $RETVAL
		fi

		echo "Copying ${STRINGS} to system area "
		echo "Please wait..."

    # list file
		echo "tar xzpf $_local_file $_list_file $_bat_file" >> $DEBUGLOG
		tar xzpf $_local_file $_list_file $_bat_file >>$DEBUGLOG 2>&1

		RETVAL=$?
		if [ "${RETVAL}" = 0 -a -f $_bat_file ]; then
			echo "autobat $_bat_file $_serverip $_user $_passwd" >>$DEBUGLOG
			/bin/sh $_bat_file $_serverip $_user $_passwd
			return 0
		fi

		if [ "${RETVAL}" = 0 -a -f $_list_file ]; then
			_ftp_list=`cat $_list_file`
			for i in $_ftp_list; do
				_key=${i%:*}
				_lfile=${i##*:}

				if [ -z "$_lfile" ]; then
					continue
				fi

				case $_key in
					KERNEL)
						get_ftp ${_lfile} ${_kern_file}
						if [ "X$RETVAL" = "X0" ]; then
							echo "$_lfile $_kern_file" >> $DEBUGLOG
							up_system_kernel $_kern_file
						fi
						;;
					RAMDISK)
						get_ftp ${_lfile} ${_ram_file}
						if [ "X$RETVAL" = "X0" ]; then
							echo "$_lfile $_ram_file" >> $DEBUGLOG
							up_system_ramdisk $_ram_file
						fi
						;;
					IPL)
						get_ftp ${_lfile} ${_ipl_file}
						if [ "X$RETVAL" = "X0" ]; then
							echo "$_lfile $_ipl_file" >> $DEBUGLOG
							up_system_ipl $_ipl_file
						fi
						;;
					ETC)
						get_ftp ${_lfile} ${_etc_file}
						if [ "X$RETVAL" = "X0" ]; then
							echo "$_lfile $_etc_file" >> $DEBUGLOG
							up_system_etc $_etc_file
						fi
						;;
					USR)
						get_ftp ${_lfile} ${_usr_file}
						if [ "X$RETVAL" = "X0" ]; then
							echo "$_lfile $_usr_file" >> $DEBUGLOG
							up_system_usr $_usr_file
						fi
						;;
					*)
						;;
				esac
			done

		else # not list file
			echo "tar xzpf $_local_file $_kern_file" >> $DEBUGLOG
			tar xzpf $_local_file $_kern_file >>$DEBUGLOG 2>&1
			if [ $? = 0 -a -f $_kern_file ]; then
				up_system_kernel $_kern_file
			fi

			echo "tar xzpf $_local_file $_ram_file" >>$DEBUGLOG
			tar xzvpf $_local_file $_ram_file >>$DEBUGLOG 2>&1
			if [ $? = 0 -a -f $_ram_file ]; then
				up_system_ramdisk $_ram_file
			fi
			
			echo "tar xzpf $_local_file $_ipl_file" >>$DEBUGLOG
			tar xzvpf $_local_file $_ipl_file >>$DEBUGLOG 2>&1
			if [ $? = 0 -a -f $_ipl_file ]; then
				up_system_ipl $_ipl_file
			fi
			
			echo "tar xzpf $_local_file $_etc_file" >>$DEBUGLOG
			tar xzvpf $_local_file $_etc_file >>$DEBUGLOG 2>&1
			if [ $? = 0 -a -f $_etc_file ]; then
				up_system_ipl $_etc_file
			fi
			
			echo "tar tzpf $_local_file $_usr_file" >>$DEBUGLOG
			tar tzvpf $_local_file $_usr_file >>$DEBUGLOG 2>&1
			if [ $? = 0 ]; then
				up_system_usr $_usr_file
			else
				rm -f $_local_file
			fi
		fi
	fi
}

RETVAL=0
if [ -z $4 -o -z $3 -o -z $2 -o -z $1 ];then
	echo "args error"
	exit 1
fi

if ! check_ppcibs ; then
	exit 0
fi

if [ -f $DEBUGLOG ]; then
	cp /dev/null $DEBUGLOG
fi

# for NetBSD
if ostype_is_netbsd ; then
	exit 0
fi

# for Linux
if ostype_is_linux ; then
	_serverip=$1
	_user=$2
	_passwd=$3
	_type=$4
	_file=$5

	if [ "X$_file" = "Xdebug" -o "X$6" = "Xdebug" ]; then
		_debug=debug
		_debugf=`basename $_file`
		FTPUPDEBUG=1
	else
		if [ "X$_file" != "X" ]; then
			_remote_file=$_file
		fi
	fi

#  ftpupdate_state on
	ln -s $0 ${LOCKFILE}
	if [ $? != 0 ]; then
		ftpupload_syslog "Already executing $CMDBAT"
		exit 1
	fi
	trap "rm -f ${LOCKFILE}" EXIT
	
	ftpup_led_update ${IBSLED_UPDATE_ON}
	ftpupload_syslog "ftpupload start"

  # $B%W%m%;%9Dd;_(B
	if [ -x /etc/rc.d/init.d/ibsexe ]; then
		/etc/rc.d/init.d/ibsexe stop
	fi

	sleep 2

	flash_rw_mount
	flash_rw_mount_usr
	if [ ! -f /tmp2/mount_ready ]; then
		sh /etc/ibsbat/mkram.bat before
	fi
	if [ ! -f /tmp3/mount_ready ]; then
		sh /etc/ibsbat/mkram7.bat before
	fi
	cd /tmp2

#  led_info_inter
	case $_type in
		ibsexe)
			up_ibsexe
			RETVAL=$?
			;;
		router)
			up_router
			RETVAL=$?
			;;
		web)
			up_web
			RETVAL=$?
			;;
		ipkt)
			if [ -z "${TAR_IPKT_COMP}" ]; then
				up_ipkt
				RETVAL=$?
			else
				up_ipkt_comp
				RETVAL=$?
			fi
			;;
		ibsgw)
			up_ibsgw
			RETVAL=$?
			;;
		pgdata)
			up_pgdata
			RETVAL=$?
			;;
		system)
			up_system
			RETVAL=$?
			if [ ${RETVAL} = 0 ]; then
				echo "Upload ${STRINGS} done."
				exit 0
			else
				echo "System update failed"
			fi
			;;
		all)
			if [ "X$_debug" = "Xdebug" ]; then
				touch /tmp/${_debugf}_to_all
			else
				up_ibsexe
				_remote_file=""
				up_router
				_remote_file=""
				up_web
				_remote_file=""
				up_ipkt
				_remote_file=""
				up_ibsgw
				_remote_file=""
				up_system
			fi
			;;
		*)
			;;
	esac

	cd /tmp
	sh /etc/ibsbat/mkram7.bat after

	sync; sync; sync
	flash_ro_mount_usr
	flash_ro_mount
	sh /etc/ibsbat/mkram.bat after
	echo "Upload ${STRINGS} done."

# LED off : update
#  ftpupdate_state off
	rm -f ${LOCKFILE}
	ftpup_led_update ${IBSLED_UPDATE_OFF}
#  led_info_off
	if [ "X$RETVAL" = "X0" ]; then
		echo "Rebooting system..."
		/sbin/init 6
	else
		echo "Update failed."
	fi
fi
