#!/bin/sh
#
# $1:conf
# $1:ppp*   $2: ip address(wan)
# $1:manual $2: ip address(wan)
# $1:boot
# $1:(none)
#

[ "X${X7DBG}" = "Xon" ] && set -x

CMDBAT=`basename $0`
NAT_BAT_VERSION=2004032701
NATP_BAT_VERSION=2004042001

#DBG_LOCAL=1

if [ -z "${DBG_LOCAL}" ]; then
	RTWEBTOP=/etc/ibsbat/Router
	NATWEBDIR=/etc/ibsbat/Router/nat
	NATDIR=/etc/ibsbat/nat
	VARDIR=/var/run
else
	RTWEBTOP=/home/ibsuser/work/Router
	NATWEBDIR=/home/ibsuser/work/nat/nat
	NATDIR=/home/ibsuser/work/nat
	VARDIR=/home/ibsuser/work/nat/var
fi

NAT=${NATWEBDIR}/nat.txt
NATTBL=${NATWEBDIR}/nat_table.txt
NATPTBL=${NATWEBDIR}/natp_table.txt

NATAWK=${NATDIR}/nat.awk
NATPAWK=${NATDIR}/natp.awk
NAT_CONF=${NATDIR}/nat.conf
NATP_CONF=${NATDIR}/natp.conf
OLDBAT_PREFIX=${NATDIR}/oldnat.

N_TO_REAL_PREFIX=${VARDIR}/ntoreal.

WANSTATE=${RTWEBTOP}/ipaddress_of_wan/wan_state.txt

IPTABLES=/sbin/iptables
LANIF=eth1
WANIF=eth0

# 2004.05.05
GLOBALIP=""

[ -f "${WANSTATE}" ] && . ${WANSTATE}

nat_syslog()
{
	logger -p kern.info -t $CMDBAT $1
}

nat_init()
{
	rm -f ${OLDBAT_PREFIX}ppp*
}

natconf_version_check()
{
	if [ ! -f "${NAT_CONF}" ]; then
		return 1    # error
	fi

	nat_conf_version=`cat ${NAT_CONF} |grep "NAT_CONF_VERSION="`
	if [ $? = 0 ]; then
		eval $nat_conf_version
		if [ "${NAT_CONF_VERSION}" -lt "${NAT_BAT_VERSION}" ]; then
			return 1   # error
		fi
	else
		return 1 # error
	fi

	return 0  # ok
}

# 2004.06.10
# network/maskǾͤȺͤ
conv_minmax()
{
	_ENTRY=$1

	_check=${_ENTRY%/*}
#echo $_ENTRY $_check
	if [ "X$_ENTRY" = "X$_check" ]; then
		return 1
	fi

	_MIN=_`/bin/ipcalc --network ${_ENTRY}`
	_MAX=_`/bin/ipcalc --broadcast ${_ENTRY}`
#  echo min:$_MIN  max:$_MAX

	_MIN_LSB=`expr ${_MIN##*.} + 1`
	_MAX_LSB=`expr ${_MAX##*.} - 1`
	_MIN_PRE=${_MIN%.*}
	_MAX_PRE=${_MAX%.*}
#  echo "  LSB:${_MIN_LSB} MAX:${_MAX_LSB}"

#  _MIN_LSB=`expr $_MIN_LSB + 1`
#  _MAX_LSB=`expr $_MAX_LSB - 1`

	_MIN_IP="${_MIN_PRE}.${_MIN_LSB}"
	_MAX_IP="${_MAX_PRE}.${_MAX_LSB}"
#  echo "  LSB:${_MIN_PRE}.${_MIN_LSB} MAX:${_MAX_PRE}.${_MAX_LSB}"
	return 0
}

create_nat_conf()
{
	nat_syslog "create_nat_conf: start"
	if [ ! -f "${NAT}" ]; then
		return 10
	fi
	cat "${NAT}" | awk -v nat_conf_version=${NAT_BAT_VERSION} -f ${NATAWK}  > ${NAT_CONF}
	if [ $? != 0 ]; then
		nat_syslog "nat_conf create error"
		return 10
	fi
	nat_syslog "nat_conf create ok"
	return 0
}

create_natp_conf()
{
	nat_syslog "create_natp_conf: start"
	if [ ! -f "${NATPTBL}" ]; then
		return 10
	fi
	cat "${NATPTBL}" | awk -v natp_conf_version=${NATP_BAT_VERSION} -f ${NATPAWK}  > ${NATP_CONF}
	if [ $? != 0 ]; then
		nat_syslog "natp_conf create error"
		return 10
	fi
	nat_syslog "natp_conf create ok"
	return 0
}

#
# $1: input interface
# $2: output interface (real :ppp0/ppp1....)
# $3: output interface (entry:ppp0/ppp1....)
set_nat_tables_masq()
{
	_input_if=$1
	_output_if=$2
	_ifname=$3

	nat_syslog "set_nat_tables_masq: start"

	if [ "X$_ifname" != "X" ]; then
		_pppoeconf=/etc/ppp/pppoe${_ifname##ppp}.conf
		[ -f ${_pppoeconf} ] && eval `grep CLAMPMSS ${_pppoeconf}`
		if [ ! -z "${CLAMPMSS}" ]; then
			CLAMPMSS1=`expr ${CLAMPMSS} + 1`
		fi
	fi

	if [ ! -z "${DBG_LOCAL}" ]; then
#    echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j MASQUERADE"
		echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j SNAT --to-source ${GLOBALIP}"
		echo "${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu"
	else
#    2004.05.05
#    echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j MASQUERADE" >>"${OLDBAT_CONF}"
#    ${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j MASQUERADE
		echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j SNAT --to-source ${GLOBALIP}" >>"${OLDBAT_CONF}"
		${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j SNAT --to-source ${GLOBALIP}

		if [ -z "$CLAMPMSS" ]; then
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu
		else
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}
		fi

		nat_syslog "masq:POST $_output_if ${LIPADDR}/${LNETMASK}"
		nat_syslog "masq:FORW $_output_if ${LIPADDR}/${LNETMASK}"
	fi
}

#
# $1: input interface
# $2: output interface (real :ppp0/ppp1....)
# $3: output interface (entry:ppp0/ppp1....)
# 2004.05.05
# $4: output ip address
#
set_nat_tables_masq_lan_all()
{
	_input_if=$1
	_output_if=$2
	_ifname=$3

	nat_syslog "set_nat_tables_masq_lan_all: start"

	if [ "X$_ifname" != "X" ]; then
		_pppoeconf=/etc/ppp/pppoe${_ifname##ppp}.conf
		[ -f ${_pppoeconf} ] && eval `grep CLAMPMSS ${_pppoeconf}`
		if [ ! -z "${CLAMPMSS}" ]; then
			CLAMPMSS1=`expr ${CLAMPMSS} + 1`
		fi
	fi

	if [ ! -z "${DBG_LOCAL}" ]; then
#    echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -j MASQUERADE"
		echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -j MASQUERADE"
		echo "${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu"
	else
#    2004.05.05
#    echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -j MASQUERADE" >> "${OLDBAT_CONF}"
#    ${IPTABLES} -t nat -A POSTROUTING -o $_output_if -j MASQUERADE
		echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -j SNAT --to-source ${GLOBALIP}" >> "${OLDBAT_CONF}"
		${IPTABLES} -t nat -A POSTROUTING -o $_output_if -j SNAT --to-source ${GLOBALIP}

		if [ -z "$CLAMPMSS" ]; then
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu
		else
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}
		fi

		nat_syslog "masq lan all:POST $_output_if"
		nat_syslog "masq lan all:FORW $_output_if"
	fi
}


#
# IP
#  㳰ϡDROP
# $1: input interface
# $2: output interface (real :ppp0/ppp1....)
# $3: output interface (entry:ppp0/ppp1....)
set_nat_tables_masq_ip()
{
	_add_list="$1"
	_output_if=$2
	_ifname=$3

	nat_syslog "set_nat_tables_masq_ip: start"

	if [ "X$_ifname" != "X" ]; then
		_pppoeconf=/etc/ppp/pppoe${_ifname##ppp}.conf
		[ -f ${_pppoeconf} ] && eval `grep CLAMPMSS ${_pppoeconf}`
		if [ ! -z "${CLAMPMSS}" ]; then
			CLAMPMSS1=`expr ${CLAMPMSS} + 1`
		fi
	fi

	if [ ! -z "${DBG_LOCAL}" ]; then
#    echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j MASQUERADE"
		echo "${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${LIPADDR}/${LNETMASK} -j SNAT --to-source ${GLOBALIP}"
		echo "${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu"
	else

		for i in $_add_list; do
			echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -s $i -j SNAT --to-source ${GLOBALIP}" >>"${OLDBAT_CONF}"
			${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s $i -j SNAT --to-source ${GLOBALIP}
		done
    # 2004.06.18 ʬȤΥѥåȤѴ
		echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -s ${GLOBALIP} -j SNAT --to-source ${GLOBALIP} " >>"${OLDBAT_CONF}"
		${IPTABLES} -t nat -A POSTROUTING -o $_output_if -s ${GLOBALIP} -j SNAT --to-source ${GLOBALIP}

		echo "${IPTABLES} -t nat \$1 POSTROUTING -o $_output_if -j DROP" >>"${OLDBAT_CONF}"
		${IPTABLES} -t nat -A POSTROUTING -o $_output_if -j DROP

		if [ -z "$CLAMPMSS" ]; then
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -j TCPMSS --clamp-mss-to-pmtu
		else
			echo "${IPTABLES} \$1 FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}" >>"${OLDBAT_CONF}"
			${IPTABLES} -I FORWARD -o $_output_if -p tcp --tcp-flags SYN,RST SYN -m tcpmss --mss ${CLAMPMSS1}: -j TCPMSS --set-mss ${CLAMPMSS}
		fi

		nat_syslog "masq:POST $_output_if ${LIPADDR}/${LNETMASK}"
		nat_syslog "masq:FORW $_output_if ${LIPADDR}/${LNETMASK}"
	fi
}

#
# $1: add/del
# $2: output interface
#
set_nat_ppp()
{
	nat_syslog "set_nat_ppp: start:$1"

	if [ -z "$1" -o -z "$2" ]; then
		return 1
	fi

	pppif=$2
	OLDBAT_CONF=${OLDBAT_PREFIX}${pppif}
	case $1 in
		del)
			if [ -f "${OLDBAT_CONF}" ]; then
				/bin/sh ${OLDBAT_CONF} -D
				rm -f ${OLDBAT_CONF}
			fi
			;;
		add)
			_ntoreal_f="${N_TO_REAL_PREFIX}${pppif}"
			if [ -f "$_ntoreal_f" ]; then
				_real_if=`cat "$_ntoreal_f"`

				_entry_type=ENTRY_${pppif}_NAT_TYPE
				_type=`eval echo \$\{$_entry_type\}`

				if [ "X$_type" = "Xnatp_auto" ]; then
					_entry_in_ip=ENTRY_${pppif}_NAT_INIP1
					_type_in_ip=`eval echo \$\{$_entry_in_ip\}`
					if [ "X$_type_in_ip" = "Xall" ]; then
						set_nat_tables_masq_lan_all ${LANIF} $_real_if $pppif
					fi
					if [ "X$_type_in_ip" = "Xlan" ]; then
						set_nat_tables_masq ${LANIF} $_real_if $pppif
					fi

        # 2004.06.09
					if [ "X$_type_in_ip" = "Xip" ]; then
						_entry_in_ip2=ENTRY_${pppif}_NAT_INIP2
						_entry_in_ip2_range=`eval echo \$\{$_entry_in_ip2\}`
						set_nat_tables_masq_ip "$_entry_in_ip2_range" $_real_if $pppif
					fi
				fi
			else # not found ntoreal file
				return 1
			fi
			;;
		*)
			;;
	esac
	return 0
}

#
# $1: add/del
#
set_nat_manual()
{
	if [ -f "${NAT_CONF}" ]; then
		. ${NAT_CONF}
	fi
	if [ -f /etc/sysconfig/network-scripts/ifcfg-${WANIF} ]; then
		. /etc/sysconfig/network-scripts/ifcfg-${WANIF}
		GLOBALIP="${IPADDR}"
	fi

#  nat_syslog "set_nat_manual: start:$1 wan_set_info:${wan_set_info} ip:${GLOBALIP}"
	nat_syslog "set_nat_manual: start:$1 pppoe:${pppoe} ip:${ip} IP:${GLOBALIP}"

	_nat_wanif=eth0
	OLDBAT_CONF=${OLDBAT_PREFIX}${_nat_wanif}

	case $1 in
		del)
			if [ -f "${OLDBAT_CONF}" ]; then
				/bin/sh ${OLDBAT_CONF} -D
				rm -f ${OLDBAT_CONF}
			fi
			;;
		add)
    # S2/S2/S2/S2/S2/S2/
    # 2005.02.27
#    if [ "X$wan_set_info" != "Xmanual" ]; then
			if [ "X$ip" != "Xecas" -a "X$ip" != "Xnormal" ]; then
				return 1
			fi

			_entry_type=ENTRY_IP_NAT_TYPE
			_type=`eval echo \$\{$_entry_type\}`
			if [ "X$_type" = "Xnatp_auto" ]; then
				_entry_in_ip=ENTRY_IP_NAT_INIP1
				_type_in_ip=`eval echo \$\{$_entry_in_ip\}`
				if [ "X$_type_in_ip" = "Xall" ]; then
					set_nat_tables_masq_lan_all ${LANIF} $_nat_wanif
				fi
				if [ "X$_type_in_ip" = "Xlan" ]; then
					set_nat_tables_masq ${LANIF} $_nat_wanif
				fi
      # 2004.06.09
				if [ "X$_type_in_ip" = "Xip" ]; then
					set_nat_tables_masq_ip "${ENTRY_IP_NAT_INIP2}" $_nat_wanif
				fi
			fi
			;;
		*)
			;;
	esac
	return 0
}

#
# 2005.04.08 Ѥʤ
# NATP
#  2005.02.27 S2
#  $1 : pppoe/manual
#
set_natp_ppp()
{
	OLDBAT_NATP=${OLDBAT_PREFIX}p.bat

	nat_syslog "set_natp_ppp: start"

	if [ -f "${NATP_CONF}" ]; then
		. ${NATP_CONF}
	fi

	if [ -f "${OLDBAT_NATP}" ]; then
		/bin/sh ${OLDBAT_NATP} -D
		rm -f ${OLDBAT_NATP}
	fi
# S2/S2/S2/S2/S2/S2/
#  2005.02.27
#  case $wan_set_info in
	case $1 in
		pppoe)
			if [ "X${pppoe}" = "Xon" ]; then
				_natp_wanif=ppp+
			fi
			;;
		manual)
			if [ "X${ip}" = "Xecas" -o "X${ip}" = "Xnormal" ]; then
				_natp_wanif=eth0
			fi
			;;
		dhcp)
			;;
		*)
			;;
	esac

	[ -z "$_natp_wanif" ] && return 10

	nat_syslog "natp wanif:$_natp_wanif"
	_entno=0
	while [ ${_entno} -le 15 ]; do
		_entry=ENTRY_NATP_${_entno}

		_natp_proto=`eval echo \$\{${_entry}_PROTOCOL\}`
		_natp_port=`eval echo \$\{${_entry}_PORT\}`
		_natp_ip=`eval echo \$\{${_entry}_IPADDRESS\}`

		nat_syslog "natp: _entno:$_entry prot:$_natp_proto port:$_natp_port ip:$_natp_ip"
		if [ -z "$_natp_proto" -o -z "$_natp_port" -o -z "$_natp_ip" ] ; then
			_entno=`expr ${_entno} + 1`
			continue
		fi

		if [ "X$_natp_proto" = "Xboth" ]; then
			_natp_args="-t nat -i $_natp_wanif -p tcp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
			_natp_args1="-t nat -i $_natp_wanif -p udp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
		else
			_natp_args="-t nat -i $_natp_wanif -p $_natp_proto --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
		fi
		if [ ! -z "${DBG_LOCAL}" ]; then
			echo "${IPTABLES} \$1 PREROUTING $_natp_args"
			if [ ! -z "$_natp_args1" ]; then
				echo "${IPTABLES} \$1 PREROUTING $_natp_args1"
			fi
		else
			echo "${IPTABLES} \$1 PREROUTING $_natp_args" >> ${OLDBAT_NATP}
			${IPTABLES} -A PREROUTING ${_natp_args}
			if [ ! -z "$_natp_args1" ]; then
				echo "${IPTABLES} \$1 PREROUTING $_natp_args1" >> ${OLDBAT_NATP}
				${IPTABLES} -A PREROUTING ${_natp_args1}
			fi
		fi

		_entno=`expr ${_entno} + 1`
		_natp_args=""
		_natp_args1=""
	done

	return 0
}

#
# NATP
# 2005.04.08 ϤȤ餺ˡwan_state.txtƤppp/eth0Ȥꤹ
#
set_natp_ppp_2()
{
	OLDBAT_NATP=${OLDBAT_PREFIX}p.bat

	nat_syslog "set_natp_ppp_2: start"

	if [ -f "${NATP_CONF}" ]; then
		. ${NATP_CONF}
	fi

	if [ -f "${OLDBAT_NATP}" ]; then
		/bin/sh ${OLDBAT_NATP} -D
		rm -f ${OLDBAT_NATP}
	fi

	if [ "X${pppoe}" = "Xon" ]; then
		_natp_wanif_ppp=ppp+
	else
		_natp_wanif_ppp=0
	fi
	if [ "X${ip}" = "Xecas" -o "X${ip}" = "Xnormal" ]; then
		_natp_wanif_eth0=eth0
	else
		_natp_wanif_eth0=0
	fi

  #2005.04.15
#  [ "$_natp_wanif_ppp" = "0" -o "$_natp_wanif_eth0" = "0" ] && return 10
  # 2006.01.10
#  [ "$_natp_wanif_ppp" = "0" -o "$_natp_wanif_eth0" = "0" ] || return 10
	[ "$_natp_wanif_ppp" = "0" -a "$_natp_wanif_eth0" = "0" ] && return 10

	nat_syslog "natp wanif:$_natp_wanif_ppp  $_natp_wanif_eth0"
	_entno=0
	while [ ${_entno} -le 15 ]; do
		_entry=ENTRY_NATP_${_entno}

		_natp_proto=`eval echo \$\{${_entry}_PROTOCOL\}`
		_natp_port=`eval echo \$\{${_entry}_PORT\}`
		_natp_ip=`eval echo \$\{${_entry}_IPADDRESS\}`

		nat_syslog "natp: _entno:$_entry prot:$_natp_proto port:$_natp_port ip:$_natp_ip"
		if [ -z "$_natp_proto" -o -z "$_natp_port" -o -z "$_natp_ip" ] ; then
			_entno=`expr ${_entno} + 1`
			continue
		fi

    # pppoe¦
		if [ "$_natp_wanif_ppp" != "0" ]; then
			if [ "X$_natp_proto" = "Xboth" ]; then
				_natp_args="-t nat -i $_natp_wanif_ppp -p tcp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
				_natp_args1="-t nat -i $_natp_wanif_ppp -p udp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
			else
				_natp_args="-t nat -i $_natp_wanif_ppp -p $_natp_proto --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
			fi
			if [ ! -z "${DBG_LOCAL}" ]; then
				echo "${IPTABLES} \$1 PREROUTING $_natp_args"
				if [ ! -z "$_natp_args1" ]; then
					echo "${IPTABLES} \$1 PREROUTING $_natp_args1"
				fi
			else
				echo "${IPTABLES} \$1 PREROUTING $_natp_args" >> ${OLDBAT_NATP}
				${IPTABLES} -A PREROUTING ${_natp_args}
				if [ ! -z "$_natp_args1" ]; then
					echo "${IPTABLES} \$1 PREROUTING $_natp_args1" >> ${OLDBAT_NATP}
					${IPTABLES} -A PREROUTING ${_natp_args1}
				fi
			fi
		fi
    # eth0¦
		if [ "$_natp_wanif_eth0" != "0" ]; then
			if [ "X$_natp_proto" = "Xboth" ]; then
				_natp_args="-t nat -i $_natp_wanif_eth0 -p tcp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
				_natp_args1="-t nat -i $_natp_wanif_eth0 -p udp --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
			else
				_natp_args="-t nat -i $_natp_wanif_eth0 -p $_natp_proto --dport $_natp_port -j DNAT --to ${_natp_ip}:${_natp_port}"
			fi
			if [ ! -z "${DBG_LOCAL}" ]; then
				echo "${IPTABLES} \$1 PREROUTING $_natp_args"
				if [ ! -z "$_natp_args1" ]; then
					echo "${IPTABLES} \$1 PREROUTING $_natp_args1"
				fi
			else
				echo "${IPTABLES} \$1 PREROUTING $_natp_args" >> ${OLDBAT_NATP}
				${IPTABLES} -A PREROUTING ${_natp_args}
				if [ ! -z "$_natp_args1" ]; then
					echo "${IPTABLES} \$1 PREROUTING $_natp_args1" >> ${OLDBAT_NATP}
					${IPTABLES} -A PREROUTING ${_natp_args1}
				fi
			fi
		fi

		_entno=`expr ${_entno} + 1`
		_natp_args=""
		_natp_args1=""
	done

	return 0
}


#
# $1: add/del
#
set_nat_ppp_all()
{
	nat_syslog "set_nat_ppp_all: start:$1"

	if [ -f "${NAT_CONF}" ]; then
		. ${NAT_CONF}
	fi

  # all
	WAN_LIST="ppp0 ppp1 ppp2 ppp3 ppp4"
	for pppif in ${WAN_LIST}; do
		set_nat_ppp $1 $pppif
	done
}

##############################
_spec=$1

if [ ! -d "${NATDIR}" ]; then mkdir "${NATDIR}"; fi

if [ -f /etc/sysconfig/network-scripts/ifcfg-${LANIF} ]; then
	. /etc/sysconfig/network-scripts/ifcfg-${LANIF}
	LIPADDR="${IPADDR}"
	LNETMASK="${NETMASK}"
fi

if [ -f "${NAT_CONF}" ]; then
	. ${NAT_CONF}
fi

RETVAL=0
case $_spec in
	boot)
		nat_syslog "$1 start"
		nat_init
		create_nat_conf
		RETVAL=$?
		if [ ${RETVAL} = 10 ]; then
			exit 10
		fi
		create_natp_conf
		if [ $? = 10 ]; then
			exit 10
		fi
		set_nat_manual add
# S2  2005.02.27
#  set_natp_ppp
# S2 2005.04.08
		set_natp_ppp_2
# 2005.04.08 ̤ѤȤ(嵭ΰΥȤ)
## 2005.04.05 IP߻ԲĤΤΥƥ
##  set_natp_ppp pppoe
##  set_natp_ppp manual
#  set_natp_ppp manual
#  set_natp_ppp pppoe
		;;
	ppp*)
		GLOBALIP=$2
		nat_syslog "$1 start:global:${GLOBALIP}"
#  set_nat_manual del		# 2005.02.27 S2Ǻ
		set_nat_ppp del $_spec
		set_nat_ppp add $_spec
		RETVAL=$?
		;;
	manual)
		GLOBALIP=$2
		nat_syslog "$1 start:global:${GLOBALIP}"
#  set_nat_ppp del $_spec	# 2005.02.27 S2Ǻ
		set_nat_manual del
		set_nat_manual add
		;;
	conf)
		nat_syslog "$1 start"
		create_nat_conf
		create_natp_conf
		;;
	*)
		nat_syslog "start"
		set_nat_manual del
		set_nat_ppp_all del

		create_nat_conf
		RETVAL=$?
		if [ ${RETVAL} = 10 ]; then
			exit 10
		fi
		create_natp_conf
		RETVAL=$?
		if [ ${RETVAL} = 10 ]; then
			exit 10
		fi

#  case $wan_set_info in	# 2005.02.27 S2ѹ
#  pppoe)
#    set_nat_ppp_all add
#    ;;
#  manual)
#    set_nat_manual add
#    ;;
#  *)
#    ;;
#  esac
		if [ "X${pppoe}" = "Xon" ]; then
			set_nat_ppp_all add
		fi
		if [ "X${ip}" = "Xnormal" -o "X${ip}" = "Xecas" ]; then
			set_nat_manual add
		fi

# S2  2005.02.27
#  set_natp_ppp
# S2 2005.04.08
		set_natp_ppp_2
# 2005.04.08 ̤ѤȤ(嵭ΰΥȤ)
## 2005.04.05 IP߻ԲĤΤΥƥ
##  set_natp_ppp pppoe
##  set_natp_ppp manual
#  set_natp_ppp manual
#  set_natp_ppp pppoe
		RETVAL=0
		;;
esac

exit $RETVAL
