#!/bin/sh

CMDBAT=`basename $0`
PATH=/usr/local/pgsql/bin:/bin:/usr/bin:/sbin:/usr/sbin

BKTOP=/mnt/usr/BACKUP
BKDIR=${BKTOP}/pgdump

BEFORECICLE=${BKTOP}/pgdump.before
STARTCICLE=${BKTOP}/pgdump.start
DATEOPT="+%Y/%m/%d-%H:%M"
LOCKFILE=${BKTOP}/${CMDBAT}.lock
RESTART=/var/log/pgsql_restart_save

PGDUMP=/usr/local/pgsql/bin/pg_dump
PSQL=/usr/local/pgsql/bin/psql

PGTOP=/var
PGDATA=data/base/ibsdatabase
DEBUGF=/tmp/pgdump.lst
#DEBUGF=/dev/null

BKFULLDIR=${BKTOP}/pgdump_full
BKFULLFILE=${BKFULLDIR}/pgdump_full.gz
COUNTF=${BKTOP}/pgdump_count
FCOUNTF=${BKTOP}/pgdump_full_count
TAR_COUNTF=${BKTOP}/pgdump_tar_count

MAXCOUNT=10

PREFIX=""

pgdump_cron_syslog()
{
  logger -p kern.info -t $CMDBAT $1
}

pg_dump_increment()
{
  RETVAL=0
  grep "/tbl" /tmp/pg_inc.txt | while read line; do
    _tbl=${line##*/}
    echo "name:$_tbl" >> ${DEBUGF}

    _fname=${PREFIX}_${_tbl}
    ${PGDUMP} -c ibsdatabase -t $_tbl > /tmp/${_fname}

    if [ -s /tmp/${_fname} ]; then
      gzip -c /tmp/${_fname} > ${BKDIR}/${_fname}.gz
    fi
    rm -f /tmp/${_fname}
    if [ -f  "$RESTART" ]; then
      pgdump_cron_syslog "Abort"
      RETVAL=2
      break
    fi
  done
  return $RETVAL
}

file_check()
{
  _newer=$1
  [ ! -f "$_newer" ] && return 1

  (cd $PGTOP;\
    find $PGDATA -type f -newer $_newer -print > /tmp/pg_inc.txt
  )
  if [ -s /tmp/pg_inc.txt ]; then
    pgdump_cron_syslog "Renew"
    return 0
  else
    pgdump_cron_syslog "Not renew"
    return 1
  fi
}

increment_save()
{
  _newer=$1
  [ ! -f "$_newer" ] && return 1

  pgdump_cron_syslog "pg_dump start:`date $DATEOPT`"
  (cd $PGTOP;\
    find $PGDATA -type f -newer $_newer -print > /tmp/pg_inc.txt
  )
  if [ -s /tmp/pg_inc.txt ]; then
    pgdump_cron_syslog "Increment"
    pg_dump_increment
  else
    pgdump_cron_syslog "Not increment"
  fi
  pgdump_cron_syslog "pg_dump end  :`date $DATEOPT`"
  return 0
}

#
# _count=0$B$O!"(Bpg_dump$B$G%U%k$K%P%C%/%"%C%W$7$?;~(B
# _count!=0$B$O!"(Bpg_dump$B$G%$%s%/%j%a%s%?%k%P%C%/%"%C%W$7$?;~(B
#
# $B%$%s%/%j%a%s%?%k(B10$B2s$G!"%U%k(Bpg_dump$B$9$k(B
#
backup_start()
{
  [ ! -d ${BKDIR} ] && mkdir -p ${BKDIR}
  [ ! -d ${BKFULLDIR} ] && mkdir -p ${BKFULLDIR}
  [ -f "$RESTART" ] && return 0
  
  ln -s $0 ${LOCKFILE} > /dev/null 2>&1
  if [ $? != 0 ]; then
    echo "already exec" >> ${DEBUGF}
    return 0
  fi
  trap  "rm -f ${LOCKFILE}" EXIT
  
  if [ -f "$COUNTF" ]; then
    _count=`cat $COUNTF`
  else
    _count=0
  fi
  _count=`expr $_count + 1`

  PREFIX=$_count

  touch ${STARTCICLE}

  RETVAL=1
#  if [ $_count -gt $MAXCOUNT ]; then
#    backup_full
#    RETVAL=$?
#    if [ $RETVAL = 0 ]; then
#      rm -f ${PKDIR}/*
#      _count=0
#    fi
#  else

    if [ ! -f "${BEFORECICLE}" ]; then
      _newerfile=/mnt/IBS/pgsql_data.tar.gz
    else
      _newerfile=${BEFORECICLE}
    fi

    file_check $_newerfile
    if [ $? = 0 ]; then
      backup_full
      RETVAL=$?
    fi
#    increment_save $_newerfile
#    RETVAL=$?
  
    if [ $RETVAL = 0 ]; then
#      echo $_count > ${COUNTF}
      mv $STARTCICLE $BEFORECICLE
      return 0
    else
      return 1
    fi
#  fi
}

#
# backup all
backup_full()
{
  [ -f ${TAR_COUNTF} ] && _tcount=`cat ${TAR_COUNTF}` || _tcount=0
  [ -f ${FCOUNTF}    ] && _fcount=`cat ${FCOUNTF}`    || _fcount=0

  [ -f ${BKFULLFILE} ] && mv ${BKFULLFILE} ${BKFULLFILE}.bak
  ${PGDUMP} -c ibsdatabase | gzip -c > ${BKFULLFILE}
  RETVAL=$?
  if [ $RETVAL = 0 ]; then
    echo "0" > ${COUNTF}
    if [ $_fcount -le $_tcount ]; then
      _fcount=`expr $_tcount + 1`
    else
      _fcount=`expr $_fcount + 1`
    fi
    echo "$_fcount" > ${FOUNTF}
  fi
  
  return $RETVAL
}

#
restore_full_pgdump()
{
  _bkfile=0
  if [ -f ${BKFULLFILE} ] ; then
    gzip -t ${BKFULLFILE} > /dev/null 2>&1
    if [ $? != 0 -a -f ${BKFULLFILE}.bak ]; then
      gzip -t ${BKFULLFILE}.bak
      if [ $? = 0 ]; then
        _bkfile=${BKFULLFILE}.bak
      fi
    elif [ $? = 0 ]; then
      _bkfile=${BKFULLFILE}
    fi
  fi

  if [ "X$_bkfile" = "X0" ]; then
    return 1
  fi
  pgdump_cron_syslog "restore_full:dropdb"
  /usr/local/pgsql/bin/dropdb ibsdatabase > /dev/null 2>&1
  pgdump_cron_syslog "restore_full:createdb"
  /usr/local/pgsql/bin/createdb ibsdatabase > /dev/null 2>&1
  pgdump_cron_syslog "restore_full:psql db"
  gunzip -c $_bkfile | ${PSQL} ibsdatabase > /dev/null 2>&1
  pgdump_cron_syslog "restore_full:vacuumdb"
  vacuumdb ibsdatabase
}

#
# $1: count
restore_sub()
{
  _rcount=$1
  LIST=`ls -1 ${PKDIR}/${_rcount}_*`
  [ -z "$LIST" ] && return 0

  for i in $LIST
  do
    _rfile=${PKDIR}/${i}
    gzip -t $_rfile > /dev/null 2>&1
    if [ $? = 0 ]; then
      pgdump_cron_syslog "restore_sub: psql $_rfile"
      gunzip -c $_rfile | ${PSQL} ibsdatabase > /dev/null 2>&1
      if [ $? = 0 ]; then
        rm -f $_rfile
      fi
    fi
  done
}

#
# $B%V!<%H;~$N$_(B
restore_start()
{
  [ -f ${TAR_COUNTF} ] && _tcount=`cat ${TAR_COUNTF}` || _tcount=0
  [ -f ${FCOUNTF}    ] && _fcount=`cat ${FCOUNTF}`    || _fcount=0
  [ -f ${COUNTF}     ] && _count=`cat ${COUNTF}`    || _count=0

  # tar$B$h$j$b(Bfull pg_dump$B$N%U%!%$%k$,$"$k;~(B
  if [ $_fcount -gt $_tcount ]; then
    restore_full_pgdump

#    # full pg_dump$B0J9_$N%$%s%/%j%a%s%?%k%U%!%$%k(B
#    _count_tmp=1
#    while [ $_count_tmp -le $_count ]; do
#      restore_sub $_count_tmp
#    done
  fi
}

_spec=$1
case $_spec in
restore)
  [ -f ${LOCKFILE} ] && rm -f ${LOCKFILE}
  restore_start
  ;;
backup)
  [ ! -f /var/lock/subsys/pgsql ] && return 0
  backup_start
  ;;
*)
  ;;
esac
