#!/bin/sh
#
# ѥ
#  $1: allow
#  $2: remote ip
#

IPTABLES=/sbin/iptables
APACHECTL=/usr/local/apache/bin/apachectl
access_conf=/usr/local/apache/conf/access.conf


remoteaccess_syslog()
{
  logger -p kern.info -t "remote" "$1"
}

#
# $1: ipaddress
allow_apache_conf()
{
  _remote_ip=$1
  _conf_f=$access_conf
  _modify_f=${access_conf}.rem

  if [ -f ${_conf_f} ]; then
    grep "#remote-access-ibs" ${_conf_f} >/dev/null 2>&1
    if [ $? = 0 ]; then
      return 2
    fi

    ipcalc -n "$_remote_ip" 255.255.255.0 > /dev/null 2>&1
    if [ $? = 0 ]; then
      _remote_ip_is_ip=1
      _remote_ip="$_remote_ip/32"
    else
      _remote_ip_is_ip=0
    fi
    grep "^#remote-ibs" ${_conf_f} > /dev/null 2>&1
    if [ $? != 0 ]; then
      cat ${_conf_f} | awk -v remoteip=$_remote_ip \
        -f /etc/ibsbat/remoteaccess1.awk > ${_modify_f}
    else
      cat ${_conf_f} | awk -v remoteip=$_remote_ip \
	-f /etc/ibsbat/remoteaccess.awk > ${_modify_f}
    fi
    if [ $? = 0 ]; then
      mv  ${_modify_f}  ${_conf_f}
#      ${APACHECTL} restart
      ${APACHECTL} graceful
      RETVAL=$?
    else
      rm -f  ${_modify_f}
    fi
  else
    return 1
  fi
  return $RETVAL
}

deny_apache_conf()
{
  _conf_f=$access_conf
  _modify_f=${access_conf}.rem

  if [ -f ${_conf_f} ]; then

    cat ${_conf_f} | sed -e 's:.*#remote-access-ibs$:#remote-ibs:' >  ${_modify_f}
    if [ $? = 0 ]; then
      mv  ${_modify_f} ${_conf_f}
#      ${APACHECTL} restart
      ${APACHECTL} graceful
      RETVAL=$?
    else
      rm -f  ${_modify_f}
    fi
  else
    return 1
  fi
  return $RETVAL
}

#
allow_iptables()
{
  [ -z "$1" ] && return 1
  _remoteip=$1
  
  ${IPTABLES} -I INPUT --source ${1}/32 -p tcp --dport 443 -j ACCEPT
  return 0
}
#
deny_iptables()
{
  [ -z "$1" ] && return 1
  _remoteip=$1
  
  ${IPTABLES} -D INPUT  --source ${1}/32 -p tcp --dport 443 -j ACCEPT
  return 0
}

_spec=$1
_remoteip=$2

[ -z "$_remoteip" ] && exit 1

case $_spec in
allow)
  remoteaccess_syslog "start"
  allow_apache_conf $_remoteip
  if [ $? = 0 ]; then
    sleep 1
    allow_iptables $_remoteip
    if [ $? != 0 ]; then
      remoteaccess_syslog "filter ng"
      deny_apache_conf $_remoteip
      RETVAL=1
    else
      remoteaccess_syslog "ok"
      RETVAL=0
    fi
  elif [ $? = 2 ]; then
    remoteaccess_syslog "already open"
    RETVAL=1
  else
    remoteaccess_syslog "apache ng"
    RETVAL=1
  fi
  ;;
deny)
  remoteaccess_syslog "stop"
  deny_iptables $_remoteip
  deny_apache_conf $_remoteip
  RETVAL=0
  ;;
*)
  ;;
esac
exit $RETVAL